#ifndef MSP_GL_EFFECT_H_
#define MSP_GL_EFFECT_H_

#include <set>
#include <msp/core/refptr.h>
#include "renderable.h"

namespace Msp {
namespace GL {

class Sampler;

/**
Effects are used to wrap other renderables and give them additional visual
properties.  An Effect's render method should set up the necessary state, call
the wrapped Renderable's render method, and clean up after itself.
*/
class Effect: public Renderable
{
protected:
	Renderable &renderable;
	std::set<Tag> enabled_passes;

protected:
	Effect(Renderable &);
public:
	virtual ~Effect() { }

	void enable_for_pass(const Tag &);
	void disable_for_pass(const Tag &);

	virtual const Matrix *get_matrix() const { return renderable.get_matrix(); }
	virtual const Geometry::BoundingSphere<float, 3> *get_bounding_sphere() const { return renderable.get_bounding_sphere(); }

	virtual void setup_frame(Renderer &r) { renderable.setup_frame(r); }
	virtual void finish_frame() { renderable.finish_frame(); }
};

} // namespace GL
} // namespace Msp

#endif
